@@ -103,7 +103,7 @@ def findUser():
             # Logged-in user (if we have a tty)
             ( quietRun( 'who am i' ).split() or [ False ] )[ 0 ] or
             # Give up and return effective user
-            quietRun( 'whoami' ) )
+            quietRun( 'whoami' ).strip() )
 
 
 class ClusterCleanup( object ):
@@ -402,7 +402,7 @@ class RemoteLink( Link ):
         cmd = 'ip link set %s netns %s' % ( intf, node.pid )
         node.rcmd( cmd )
         links = node.cmd( 'ip link show' )
-        if not ' %s:' % intf in links:
+        if not re.search( r' %s[:@]' % intf, links ):
             if printError:
                 error( '*** Error: RemoteLink.moveIntf: ' + intf +
                        ' not successfully moved to ' + node.name + '\n' )
