@@ -5,13 +5,12 @@ Test for cpu.py
 
 results format:
 
-    sched   cpu client MB/s
-
-    cfs 45.00%  13254.669841
-    cfs 40.00%  11822.441399
-    cfs 30.00%  5112.963009
-    cfs 20.00%  3449.090009
-    cfs 10.00%  2271.741564
+sched	cpu	received bits/sec
+cfs	50%	8.14e+09
+cfs	40%	6.48e+09
+cfs	30%	4.56e+09
+cfs	20%	2.84e+09
+cfs	10%	1.29e+09
 
 """
 
@@ -26,13 +25,13 @@ class testCPU( unittest.TestCase ):
     @unittest.skipIf( '-quick' in sys.argv, 'long test' )
     def testCPU( self ):
         "Verify that CPU utilization is monotonically decreasing for each scheduler"
-        p = pexpect.spawn( 'python -m mininet.examples.cpu' )
+        p = pexpect.spawn( 'python -m mininet.examples.cpu', timeout=300 )
         # matches each line from results( shown above )
-        opts = [ '([a-z]+)\t([\d\.]+)%\t([\d\.]+)',
+        opts = [ '([a-z]+)\t([\d\.]+)%\t([\d\.e\+]+)',
                  pexpect.EOF ]
         scheds = []
         while True:
-            index = p.expect( opts, timeout=600 )
+            index = p.expect( opts )
             if index == 0:
                 sched = p.match.group( 1 )
                 cpu = float( p.match.group( 2 ) )
@@ -40,7 +39,9 @@ class testCPU( unittest.TestCase ):
                 if sched not in scheds:
                     scheds.append( sched )
                 else:
-                    self.assertTrue( bw < previous_bw )
+                    self.assertTrue( bw < previous_bw,
+                                     "%e should be less than %e\n" %
+                                     ( bw, previous_bw ) )
                 previous_bw = bw
             else:
                 break
