@@ -14,7 +14,7 @@ class testScratchNet( unittest.TestCase ):
     def pingTest( self, name ):
         "Verify that no ping packets were dropped"
         p = pexpect.spawn( 'python -m %s' % name )
-        index = p.expect( self.opts )
+        index = p.expect( self.opts, timeout=120 )
         self.assertEqual( index, 0 )
 
     def testPingKernel( self ):
